#include "Part2.h"

//     
void addSubjectToFile(const std::string& a_filename)
{
	std::ofstream binFile(a_filename, std::ios::binary);

	if (!binFile.is_open())
	{
		throw FileError();
	}
	Subject newElement;
	
	std::cin >> newElement;
	binFile.write(reinterpret_cast<const char*>(&newElement), sizeof(Subject));
}

void createMap(const std::string& a_filename, std::map<int, Subject>& a_map)
{
	std::ifstream binFile(a_filename, std::ios::binary);
	if (!binFile)
	{
		throw FileError();
	}

	Subject temp;
	while (binFile.read(reinterpret_cast<char*>(&temp), sizeof(Subject)))
	{
		a_map.insert(std::make_pair(temp.mId, temp));
	}
}


//   
void transformToTxt(const std::string& a_filenameBin, const std::string& a_filenameTxt)
{
	std::ofstream txtFile(a_filenameTxt);
	if (!txtFile.is_open())
	{
		throw FileError();
	}
	txtFile << std::setw(3) << "Subject Name" << std::setw(3) << "|" <<
		std::setw(3) << "Teacher" << std::setw(3) << "|" <<
		std::setw(3) << "Subject id" << std::setw(3) << "|" <<
		std::setw(3) << "Group id" << std::setw(3) << "|" <<
		std::setw(3) << "Number of students" << std::setw(3) << "|" <<
		std::setw(3) << "Lecture hours" << std::setw(3) << "|" <<
		std::setw(3) << "Practice hours" << std::setw(3) << "|" <<
		std::setw(3) << "Exam" << std::setw(3) << "|" <<
		std::setw(3) << "CourseWork" << std::setw(3) << "|" << std::endl;
	std::map<int, Subject> map;
	createMap(a_filenameBin, map);

	for (const auto& i : map)
	{
		txtFile << std::setw(3) << i.second.mName   << std::setw(6) << "|" <<
			std::setw(3) << i.second.mTeacher       << std::setw(6) << "|" <<
			std::setw(3) << i.second.mId            << std::setw(6) << "|" <<
			std::setw(3) << i.second.mGroupId       << std::setw(6) << "|" <<
			std::setw(3) << i.second.mGroupCount    << std::setw(6) << "|" <<
			std::setw(3) << i.second.mLecturesHours << std::setw(6) << "|" <<
			std::setw(3) << i.second.mPracticeHours << std::setw(6) << "|" <<
			std::setw(3) << i.second.mExam          << std::setw(6) << "|" <<
			std::setw(3) << i.second.mCourseWork    << std::setw(6) << "|" << std::endl;
	}
}

// 
void deleteSubject(const std::string& a_filename, int a_id)
{
	std::ofstream binFile(a_filename, std::ios::binary);
	if (!binFile)
	{
		throw FileError();
	}

	std::map<int, Subject> map;
	createMap(a_filename, map);
	map.erase(a_id);
	for (const auto& i : map)
	{
		binFile.write(reinterpret_cast<const char*>(&i.second), sizeof(Subject));
	}
}

void changeSubject(const std::string& a_filename, int a_id, const Subject& a_newElem)
{
	std::ofstream binFile(a_filename, std::ios::binary);
	if (!binFile)
	{
		throw FileError();
	}
	std::map<int, Subject> map;
	createMap(a_filename, map);
	map[a_id] = a_newElem;
	for (const auto& i : map)
	{
		binFile.write(reinterpret_cast<const char*>(&i.second), sizeof(Subject));
	}
}



// 

//teacherName
void findByTeacher(const std::string& a_filename, const std::string& a_resultFile, const std::string& a_teacherName)
{
	std::map<int, Subject> map;
	createMap(a_filename, map);
	std::vector<int> result;

	std::for_each(map.begin(), map.end(), [a_teacherName, &result](const std::pair<int, Subject>& i)
		{
			if (i.second.mTeacher == a_teacherName)
			{
				result.push_back(i.first);
			}
		});

	std::ofstream resFile(a_resultFile, std::ios::binary);
	if (!resFile)
	{
		throw FileError();
	}
	for (const auto& i : result)
	{
		resFile.write(reinterpret_cast<const char*>(&map[i]), sizeof(Subject));
	}
}

//groupId
void findByGroupId(const std::string& a_filename, const std::string& a_resultFile, int a_groupId)
{
	std::map<int, Subject> map;
	createMap(a_filename, map);
	std::vector<int> result;

	std::for_each(map.begin(), map.end(), [a_groupId, &result](const std::pair<int, Subject>& i)
		{
			if (i.second.mGroupId == a_groupId)
			{
				result.push_back(i.first);
			}
		});

	std::ofstream resFile(a_resultFile, std::ios::binary);
	if (!resFile)
	{
		throw FileError();
	}
	for (const auto& i : result)
	{
		resFile.write(reinterpret_cast<const char*>(&map[i]), sizeof(Subject));
	}
}

//courseWork
void findByCourseWork(const std::string& a_filename, const std::string& a_resultFile, bool a_courseWork)
{
	std::map<int, Subject> map;
	createMap(a_filename, map);
	std::vector<int> result;

	std::for_each(map.begin(), map.end(), [a_courseWork, &result](const std::pair<int, Subject>& i)
		{
			if (i.second.mCourseWork == a_courseWork)
			{
				result.push_back(i.first);
			}
		});

	std::ofstream resFile(a_resultFile, std::ios::binary);
	if (!resFile)
	{
		throw FileError();
	}
	for (const auto& i : result)
	{
		resFile.write(reinterpret_cast<const char*>(&map[i]), sizeof(Subject));
	}
}

//exam
void findByExam(const std::string& a_filename, const std::string& a_resultFile, bool a_exam)
{
	std::map<int, Subject> map;
	createMap(a_filename, map);
	std::vector<int> result;

	std::for_each(map.begin(), map.end(), [a_exam, &result](const std::pair<int, Subject>& i)
		{
			if (i.second.mExam == a_exam)
			{
				result.push_back(i.first);
			}
		});

	std::ofstream resFile(a_resultFile, std::ios::binary);
	if (!resFile)
	{
		throw FileError();
	}
	for (const auto& i : result)
	{
		resFile.write(reinterpret_cast<const char*>(&map[i]), sizeof(Subject));
	}
}

void testPart2()
{
	const std::string binFile1 = "binFile1.bin";
	const std::string txtFile = "txtFile.txt";
	const std::string resultFile = "resultFile.bin";

	bool block = true;
	int id;
	Subject newElem;
	bool temp;
	std::string tempStr;

	try
	{
		while (block)
		{
			int select;
			std::cout << "1. Create Subject\n" <<
				"2. Transform Binary file to Txt file\n" <<
				"3. Delete Subject\n" <<
				"4. Change Subject\n" <<
				"5. Find Subjects by Teacher's name\n" <<
				"6. Find Subjects by group id\n" <<
				"7. Find Subjects by coursework availability\n" <<
				"8. Find Subjects by exam availability\n" <<
				"9. Exit\n";

			std::cout << "Select option:" << std::endl;
			std::cin >> select;

			switch (select)
			{
			case 1:
				addSubjectToFile(binFile1);
				break;
			case 2:
				transformToTxt(binFile1, txtFile);
				break;
			case 3:
				std::cout << "Enter id of subject to delete:\n" << std::endl;
				std::cin >> id;
				deleteSubject(binFile1, id);
				break;
			case 4:
				std::cout << "Enter id of subject you want to change:\n" << std::endl;
				std::cin >> id;
				std::cout << "Enter the new subject data:\n" << std::endl;
				std::cin >> newElem;
				changeSubject(binFile1, id, newElem);
				break;
			case 5:
				std::cout << "Enter the Teacher's name\n" << std::endl;
				std::cin >> tempStr;
				findByTeacher(binFile1, resultFile, tempStr);
				break;
			case 6:
				std::cout << "Enter the group id:\n" << std::endl;
				std::cin >> id;
				findByGroupId(binFile1, resultFile, id);
				break;
			case 7:
				std::cout << "Enter 1 in case of availability of coursework, other - 0\n" << std::endl;
				std::cin >> temp;
				findByCourseWork(binFile1, resultFile, temp);
				break;
			case 8:
				std::cout << "Enter 1 in case of availability of exam, other - 0\n" << std::endl;
				std::cin >> temp;
				findByExam(binFile1, resultFile, temp);
				break;
			case 9:
				block = false;
				break;
			default:
				std::cout << "Error, try again\n" << std::endl;
				break;
			}
		}
	}
	catch (const std::exception& error)
	{
		std::cout << error.what();
	}
}